<?php

namespace Drupal\Tests\marketing_cloud_platform\Functional;

use League\JsonGuard;
use League\JsonReference;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests the base marketing_cloud_platform module.
 *
 * @group marketing_cloud
 * @runTestsInSeparateProcesses
 * @preserveGlobalState disabled
 */
class MarketingCloudPlatformTest extends BrowserTestBase {

  /**
   * Modules to install.
   *
   * @var array
   */
  public static $modules = ['marketing_cloud', 'marketing_cloud_platform'];

  protected $adminUser;
  protected $service;
  protected $metaSchema;
  protected $moduleConfig;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    // Create user.
    $this->adminUser = $this->drupalCreateUser(['administer_marketing_cloud']);
    $this->drupalLogin($this->adminUser);
    // Set module config.
    $this->config('marketing_cloud.settings')
      ->set('client_id', 'testingid')
      ->set('client_secret', 'testingsecret')
      ->set('validate_json', TRUE)
      ->set('validate_schema', TRUE)
      ->set('do_not_send', TRUE)
      ->save();
    // Get schema validator.
    $this->metaSchema = JsonReference\Dereferencer::draft6()->dereference('http://json-schema.org/draft-06/schema#');
    // Create service.
    $this->service = \Drupal::service('marketing_cloud_platform.service');
    // Get config object.
    $this->moduleConfig = \Drupal::config('marketing_cloud_platform.settings');
  }

  /**
   * Tests the services and schemas for marketing_cloud_platform.
   */
  public function testDefinitions() {
    /*
     * Test getEndpoints
     */
    $this->validateDefinition('get_endpoints', 'get');
    // Validate service against expected inputs.
    $result = $this->service
      ->getEndpoints();
    $this->assertNotFalse($result, 'Unable to parse the input data in getEndpoints().');
    $this->assertEquals(
      [
        'url' => 'https://www.exacttargetapis.com/platform/v1/endpoints',
        'data' => '{}',
      ],
      $result, 'Unexpected request data generated by getEndpoints()'
    );

    /*
     * Test getTokenContext
     */
    $this->validateDefinition('get_token_context', 'get');
    // Validate service against expected inputs.
    $result = $this->service
      ->getTokenContext();
    $this->assertNotFalse($result, 'Unable to parse the input data in getTokenContext().');
    $this->assertEquals(
      [
        'url' => 'https://www.exacttargetapis.com/platform/v1/tokenContext',
        'data' => '{}',
      ],
      $result, 'Unexpected request data generated by getTokenContext()'
    );

    /*
     * Test getEndpoint
     */
    $this->validateDefinition('get_endpoint', 'get');
    // Validate service against expected inputs.
    $result = $this->service
      ->getEndpoint('soap');
    $this->assertNotFalse($result, 'Unable to parse the input data in getEndpoint().');
    $this->assertEquals(
      [
        'url' => 'https://www.exacttargetapis.com/platform/v1/endpoints/soap',
        'data' => '{}',
      ],
      $result, 'Unexpected request data generated by getEndpoint()'
    );
  }

  /**
   * Test that the Json-Schema is valid, and that the API method id correct.
   *
   * @param string $machineName
   *   The machine name for the api call definition.
   * @param string $expectedMethod
   *   The expected api call method.
   */
  protected function validateDefinition($machineName, $expectedMethod) {
    // Validate schema.
    $method = $this->moduleConfig->get("definitions.$machineName.method");
    $this->assertEquals($expectedMethod, $method, "bad method for $machineName definition");
    $schema = $this->moduleConfig->get("definitions.$machineName.schema");
    $this->assertNotEmpty($schema, "json schema for $machineName definition is empty.");
    $schema = json_decode($schema);
    $this->assertNotEmpty($schema, "json schema for $machineName definition is invalid json.");
    $validator = new JsonGuard\Validator($schema, $this->metaSchema);
    $this->assertNotFalse($validator->passes(), "schema validation failed for $machineName.");
  }

}
